"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigProcessor = void 0;
var ajv_1 = __importDefault(require("ajv"));
var configuration_schema_1 = require("./model/configuration/configuration.schema");
var ConfigProcessor = /** @class */ (function () {
    function ConfigProcessor(json) {
        var ajv = new ajv_1.default();
        this._validatorFunction = ajv.compile(configuration_schema_1.ConfigurationSchema.schema);
        this._json = json;
    }
    ConfigProcessor.prototype.validate = function () {
        return this._validatorFunction(this._json);
    };
    ConfigProcessor.prototype.compile = function () {
        if (this.validate()) {
            return {
                data: this.mergeLocalAndGlobal(this._json),
                errors: undefined,
            };
        }
        return {
            data: undefined,
            errors: this._validatorFunction.errors,
        };
    };
    ConfigProcessor.prototype.mergeLocalAndGlobal = function (configuration) {
        var mergedEndpoints = Object.entries(configuration.endpoints)
            .map(function (_a) {
            var _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7;
            var path = _a[0], endpointConfig = _a[1];
            var diffIgnoreKeys = (_c = (_b = endpointConfig === null || endpointConfig === void 0 ? void 0 : endpointConfig.options) === null || _b === void 0 ? void 0 : _b.diff) === null || _c === void 0 ? void 0 : _c.ignoreKeys;
            var diffSortBy = (_e = (_d = endpointConfig === null || endpointConfig === void 0 ? void 0 : endpointConfig.options) === null || _d === void 0 ? void 0 : _d.diff) === null || _e === void 0 ? void 0 : _e.sortBy;
            return {
                key: path,
                value: __assign(__assign({}, endpointConfig), { substitutions: __assign(__assign({}, (((_g = (_f = configuration.configuration) === null || _f === void 0 ? void 0 : _f.global) === null || _g === void 0 ? void 0 : _g.substitutions) || {})), (endpointConfig.substitutions || {})), headers: __assign(__assign({}, (((_j = (_h = configuration.configuration) === null || _h === void 0 ? void 0 : _h.global) === null || _j === void 0 ? void 0 : _j.headers) || {})), (endpointConfig.headers || {})), options: {
                        diff: {
                            enabled: ((_l = (_k = endpointConfig === null || endpointConfig === void 0 ? void 0 : endpointConfig.options) === null || _k === void 0 ? void 0 : _k.diff) === null || _l === void 0 ? void 0 : _l.enabled) === undefined
                                ? (_r = (_q = (_p = (_o = (_m = configuration.configuration) === null || _m === void 0 ? void 0 : _m.global) === null || _o === void 0 ? void 0 : _o.options) === null || _p === void 0 ? void 0 : _p.diff) === null || _q === void 0 ? void 0 : _q.enabled) !== null && _r !== void 0 ? _r : true
                                : endpointConfig.options.diff.enabled,
                            ignoreKeys: !diffIgnoreKeys || (diffIgnoreKeys === null || diffIgnoreKeys === void 0 ? void 0 : diffIgnoreKeys.length) > 0
                                ? ConfigProcessor.distinctArray(__spreadArray(__spreadArray([], (((_v = (_u = (_t = (_s = configuration.configuration) === null || _s === void 0 ? void 0 : _s.global) === null || _t === void 0 ? void 0 : _t.options) === null || _u === void 0 ? void 0 : _u.diff) === null || _v === void 0 ? void 0 : _v.ignoreKeys) || []), true), (((_x = (_w = endpointConfig === null || endpointConfig === void 0 ? void 0 : endpointConfig.options) === null || _w === void 0 ? void 0 : _w.diff) === null || _x === void 0 ? void 0 : _x.ignoreKeys) || []), true))
                                : [],
                            sortArrays: ((_z = (_y = endpointConfig === null || endpointConfig === void 0 ? void 0 : endpointConfig.options) === null || _y === void 0 ? void 0 : _y.diff) === null || _z === void 0 ? void 0 : _z.sortArrays) === undefined
                                ? (_3 = (_2 = (_1 = (_0 = configuration.configuration) === null || _0 === void 0 ? void 0 : _0.global) === null || _1 === void 0 ? void 0 : _1.options) === null || _2 === void 0 ? void 0 : _2.diff) === null || _3 === void 0 ? void 0 : _3.sortArrays
                                : endpointConfig.options.diff.sortArrays,
                            sortBy: !!diffSortBy && diffSortBy.length >= 0 ? diffSortBy : (_7 = (_6 = (_5 = (_4 = configuration.configuration) === null || _4 === void 0 ? void 0 : _4.global) === null || _5 === void 0 ? void 0 : _5.options) === null || _6 === void 0 ? void 0 : _6.diff) === null || _7 === void 0 ? void 0 : _7.sortBy,
                        },
                    } }),
            };
        })
            .reduce(function (prev, next) {
            prev[next.key] = next.value;
            return prev;
        }, {});
        return __assign(__assign({}, configuration), { endpoints: mergedEndpoints });
    };
    ConfigProcessor.distinctArray = function (array) {
        return array.filter(function (n, i) { return array.indexOf(n) === i; });
    };
    return ConfigProcessor;
}());
exports.ConfigProcessor = ConfigProcessor;
