"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HTMLReportGenerator = void 0;
var handlebars_1 = __importDefault(require("handlebars"));
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var diff_utils_1 = require("./diff-utils");
var HTMLReportGenerator = /** @class */ (function () {
    function HTMLReportGenerator(results, config) {
        var _a, _b, _c;
        var themeFiles = ((_c = (_b = (_a = config.configuration.global) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.htmlReport) === null || _c === void 0 ? void 0 : _c.theme) === "dark"
            ? ["css/bootstrap/bootstrap.dark.min.css", "css/overrides.dark.css"]
            : ["css/bootstrap/bootstrap.flat.min.css"];
        this._template = this.readFilesAndCombine.apply(this, __spreadArray(__spreadArray(["top.html"], themeFiles, false), ["css/html.css",
            "css/annotated.css",
            "css/html-report.css",
            "close-style-head.html",
            "html-report.html",
            "bottom.html"], false));
        this._results = results;
    }
    HTMLReportGenerator.prototype.createReport = function (path) {
        var template = handlebars_1.default.compile(this._template);
        var result = template(this.transformReport(this._results));
        fs_1.default.writeFileSync(path, result);
    };
    HTMLReportGenerator.prototype.transformReport = function (data) {
        return {
            results: Object.entries(data).map(function (_a, index) {
                var _b, _c, _d, _e, _f;
                var key = _a[0], value = _a[1];
                var isJson = (_b = value.result.responseBody.metadata) === null || _b === void 0 ? void 0 : _b.isJson;
                var controlBodyDiffSettingsApplied = isJson && ((_d = (_c = value.config.options) === null || _c === void 0 ? void 0 : _c.diff) === null || _d === void 0 ? void 0 : _d.enabled)
                    ? JSON.stringify(diff_utils_1.DiffUtils.format(value.result.responseBody.control, value.config.options))
                    : null;
                var candidateBodyDiffSettingsApplied = isJson && ((_f = (_e = value.config.options) === null || _e === void 0 ? void 0 : _e.diff) === null || _f === void 0 ? void 0 : _f.enabled)
                    ? JSON.stringify(diff_utils_1.DiffUtils.format(value.result.responseBody.candidate, value.config.options))
                    : null;
                return __assign(__assign({ id: index, path: key }, value.result), { responseTime: __assign(__assign({}, value.result.responseTime), { increase: value.result.responseTime.control < value.result.responseTime.candidate }), diff: {
                        control: controlBodyDiffSettingsApplied,
                        candidate: candidateBodyDiffSettingsApplied,
                        match: controlBodyDiffSettingsApplied === candidateBodyDiffSettingsApplied,
                        error: !isJson,
                    } });
            }),
        };
    };
    HTMLReportGenerator.prototype.readFilesAndCombine = function () {
        var paths = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            paths[_i] = arguments[_i];
        }
        return paths.reduce(function (prev, next) {
            var data = fs_1.default.readFileSync(path_1.default.resolve(__dirname, "./resources/html-report/", next), "utf-8");
            return "" + prev + data;
        }, "");
    };
    return HTMLReportGenerator;
}());
exports.HTMLReportGenerator = HTMLReportGenerator;
