#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var commander_1 = require("commander");
var logger_1 = require("./logger");
var config_processor_1 = require("./config-processor");
var service_comparator_1 = require("./service-comparator");
var html_report_generator_1 = require("./html-report-generator");
var date_fns_1 = require("date-fns");
var pLimit = require("p-limit");
var clear = require("clear");
var packageJson = require("./package.json");
var exampleConfig = require("./resources/mibration-buddy-config.json");
var fs = require("fs");
var cliProgress = require("cli-progress");
var clipboardy = require("clipboardy");
var program = new commander_1.Command();
clear();
logger_1.Logger.banner("Migration Buddy");
logger_1.Logger.logColour("white", "Version: " + packageJson.version + ", Description: " + packageJson.description);
logger_1.Logger.linebreak();
program
    .name("compare")
    .version(packageJson.version)
    .command("compare <config-file>", { isDefault: true })
    .description("Run endpoint comparison using the supplied comparison")
    .option("-ohf, --output-html-file <path>", "Path to create output HTML file")
    .option("-ojf, --output-json-file <path>", "Path to create output JSON file")
    .option("--include-timestamp-in-file-name", "Include a timestamp in the file name")
    .option("-c, --concurrency <concurrency>", "How many concurrent endpoints to process", function (concurrentString) { return Number.parseInt(concurrentString); }, 1)
    .option("-oc, --output-to-clipboard", "Output results to clipboard")
    .option("-v, --verbose", "Enable verbose logging / responses")
    .action(function (configFilePath, options) { return __awaiter(void 0, void 0, void 0, function () {
    var configValidator, compiledConfig, message, serviceComparator, resultMapWithConfig, progressBar, endpoints, limit, limitedPromiseFunctions, results, outputFile, outputFile, htmlReportGenerator;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                configValidator = new config_processor_1.ConfigProcessor(readConfigFile(configFilePath, options.verbose));
                compiledConfig = configValidator.compile();
                if (!compiledConfig.data) {
                    message = "Configuration failed schema validation!";
                    if (options.verbose) {
                        logger_1.Logger.error(message, compiledConfig.errors);
                    }
                    else {
                        logger_1.Logger.error(message);
                    }
                    process.exit(-1);
                }
                logger_1.Logger.info("Starting now! -- Using a concurrency setting of: '" + options.concurrency + "'");
                serviceComparator = new service_comparator_1.ServiceComparator(compiledConfig.data);
                resultMapWithConfig = {};
                progressBar = new cliProgress.SingleBar({}, cliProgress.Presets.shades_classic);
                endpoints = compiledConfig.data.endpoints;
                progressBar.start(Object.keys(endpoints).length, 0);
                limit = pLimit(options.concurrency);
                limitedPromiseFunctions = Object.entries(endpoints).map(function (_a) {
                    var endpointPath = _a[0], endpointConfig = _a[1];
                    return limit(function () { return __awaiter(void 0, void 0, void 0, function () {
                        var _a, _b;
                        var _c;
                        return __generator(this, function (_d) {
                            switch (_d.label) {
                                case 0:
                                    _a = resultMapWithConfig;
                                    _b = endpointPath;
                                    _c = {};
                                    return [4 /*yield*/, serviceComparator.compare(endpointPath, endpointConfig)];
                                case 1:
                                    _a[_b] = (_c.result = _d.sent(),
                                        _c.config = endpointConfig,
                                        _c);
                                    progressBar.increment();
                                    return [2 /*return*/];
                            }
                        });
                    }); });
                });
                return [4 /*yield*/, Promise.all(limitedPromiseFunctions)];
            case 1:
                _a.sent();
                progressBar.stop();
                results = Object.entries(resultMapWithConfig)
                    .map(function (_a) {
                    var key = _a[0], value = _a[1];
                    return { key: key, value: value.result };
                })
                    .reduce(function (prev, next) {
                    prev[next.key] = next.value;
                    return prev;
                }, {});
                if (!!options.outputToClipboard || !!options.outputJsonFile || !!options.outputHtmlFile) {
                    if (!!options.outputToClipboard) {
                        clipboardy.writeSync(JSON.stringify(results, null, 2));
                        logger_1.Logger.info("Results copied to clipboard");
                    }
                    if (!!options.outputJsonFile) {
                        outputFile = options.outputJsonFile;
                        if (options.includeTimestampInFileName) {
                            outputFile = outputFile + "-" + (0, date_fns_1.format)(new Date(), "dd-MM-yyyy'T'HHmm");
                        }
                        if (!outputFile.endsWith(".json")) {
                            outputFile = outputFile + ".json";
                        }
                        fs.writeFileSync(outputFile, JSON.stringify(results, null, 2));
                        logger_1.Logger.info("Results written to file: " + outputFile);
                    }
                    if (!!options.outputHtmlFile) {
                        outputFile = options.outputHtmlFile;
                        if (options.includeTimestampInFileName) {
                            outputFile = outputFile + "-" + (0, date_fns_1.format)(new Date(), "dd-MM-yyyy'T'HHmm");
                        }
                        if (!outputFile.endsWith(".html")) {
                            outputFile = outputFile + ".html";
                        }
                        htmlReportGenerator = new html_report_generator_1.HTMLReportGenerator(resultMapWithConfig, compiledConfig.data);
                        htmlReportGenerator.createReport(outputFile);
                        logger_1.Logger.info("Results written to file: " + outputFile);
                    }
                }
                else {
                    logger_1.Logger.linebreak();
                    logger_1.Logger.log("Results:");
                    logger_1.Logger.log(results);
                }
                return [2 /*return*/];
        }
    });
}); });
program
    .command("generate <destination>")
    .description("Generate example configuration file to use as template")
    .action(function (destination) {
    if (!destination || !destination.endsWith(".json")) {
        logger_1.Logger.error("Destination not valid, must be a valid file path ending in '.json'.");
        return;
    }
    fs.writeFileSync(destination, JSON.stringify(exampleConfig, null, 2));
    logger_1.Logger.info("Created configuration file: " + destination);
});
if (process.argv.length == 2) {
    program.outputHelp();
}
program.parse(process.argv);
function readConfigFile(path, verboseLogging) {
    if (verboseLogging === void 0) { verboseLogging = false; }
    try {
        return JSON.parse(fs.readFileSync(path));
    }
    catch (e) {
        var message = "Error reading config file. Ensure file exists and is valid JSON.";
        if (verboseLogging) {
            logger_1.Logger.errorWithStack(message, e);
        }
        else {
            logger_1.Logger.error(message);
        }
        process.exit(-1);
    }
}
